/***************************************************************************
                           options.cpp
                           version 0.5.6 $Revision: 1.35 $
                           -------------------------------
    copyright            : (C) 1999-2003 by Stefan Preis
                         : (C) 2004-2008 Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2009-12-20 20:05:49 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   provides an user interface to change various settings of the program
 ***************************************************************************/

#include <QLayout>
#include <QTreeWidget>
#include <QStringList>
#include <QDialogButtonBox>

#include "gbsarea.h"
#include "options.h"
#include "resources.h"


optionsDialog::optionsDialog(QWidget* parent): QDialog(parent)
{
    setWindowTitle(tr("User preferences"));
    setObjectName("optionsDialog");

    QVBoxLayout* mainLayout = new QVBoxLayout;
    mainLayout->setObjectName("mainLayout");
    tabWidget = new QTabWidget();
    mainLayout->addWidget(tabWidget);

    setupLayoutTab();
    setupElementTab();
    setupDigitalTab();
    setupFeedbackTab();
    setupFeedbackTypeTab();
    setupGenericMessagesTab();

    QDialogButtonBox* buttonBox = new QDialogButtonBox(
            QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

    mainLayout->addWidget(buttonBox);
    setLayout(mainLayout);
}


void optionsDialog::setupLayoutTab()
{
    QWidget *w = new QWidget();
    QVBoxLayout* tabL = new QVBoxLayout(w);
    tabL->setSpacing(10);

    // new layout groupbox
    QGroupBox* newlayoutGB = new QGroupBox(
            tr("Default dimensions for new layouts"));
    newlayoutGB->setObjectName("newlayoutGB");
    tabL->addWidget(newlayoutGB);
    QVBoxLayout* newlayoutGBL = new QVBoxLayout();
    newlayoutGBL->setSpacing(6);
    newlayoutGB->setLayout(newlayoutGBL);

    // line with cols number spin box
    QHBoxLayout* dimcolLayout = new QHBoxLayout();
    newlayoutGBL->addLayout(dimcolLayout);
    QLabel* label = new QLabel(tr("&Columns:"));
    dimcolLayout->addWidget(label);
    dimcolLayout->addStretch();
    sbDefaultCols = new QSpinBox(newlayoutGB);
    sbDefaultCols->setMinimum(MIN_COLS);
    sbDefaultCols->setMaximum(MAX_COLS);
    dimcolLayout->addWidget(sbDefaultCols);
    sbDefaultCols->setWrapping(true);
    label->setBuddy(sbDefaultCols);

    // line with rows number spin box
    QHBoxLayout* dimrowLayout = new QHBoxLayout();
    newlayoutGBL->addLayout(dimrowLayout);
    label = new QLabel(tr("&Rows:"), newlayoutGB);
    dimrowLayout->addWidget(label);
    dimrowLayout->addStretch();
    sbDefaultRows = new QSpinBox(newlayoutGB);
    sbDefaultRows->setMinimum(MIN_ROWS);
    sbDefaultRows->setMaximum(MAX_ROWS);
    dimrowLayout->addWidget(sbDefaultRows);
    sbDefaultRows->setWrapping(true);
    label->setBuddy(sbDefaultRows);


    // external program groupbox
    QGroupBox* extprogGB = new QGroupBox(tr("External programs"));
    extprogGB->setObjectName("extprogGB");
    tabL->addWidget(extprogGB);
    QVBoxLayout* extprogGBL = new QVBoxLayout();
    extprogGBL->setSpacing(6);
    extprogGB->setLayout(extprogGBL);

    // line with editor cb
    QHBoxLayout* editorLayout = new QHBoxLayout();
    extprogGBL->addLayout(editorLayout);
    label = new QLabel(tr("&Editor for layout and preferences files:"),
            extprogGB);
    editorLayout->addWidget(label);
    editorLayout->addStretch();
    coboEditor = new QComboBox(extprogGB);
    coboEditor->addItem("kwrite");
    coboEditor->addItem("kedit");
    coboEditor->addItem("nedit");
    coboEditor->addItem("xterm -e vim");
    coboEditor->setMinimumWidth(120); //MAGIC
    label->setBuddy(coboEditor);
    editorLayout->addWidget(coboEditor);

    // line with browser cb
    QHBoxLayout* browserLayout = new QHBoxLayout();
    extprogGBL->addLayout(browserLayout);
    label = new QLabel(tr("&Browser for documentation:"), extprogGB);
    browserLayout->addWidget(label);
    browserLayout->addStretch();
    coboBrowser = new QComboBox(extprogGB);
    coboBrowser->addItem("firefox");
    coboBrowser->addItem("konqueror");
    coboBrowser->addItem("mozilla");
    coboBrowser->addItem("netscape");
    coboBrowser->addItem("opera");
    coboBrowser->addItem("xterm -e links");
    coboBrowser->addItem("xterm -e lynx");
    coboBrowser->addItem("xterm -e w3m");
    coboBrowser->setMinimumWidth(120); //MAGIC
    label->setBuddy(coboBrowser);
    browserLayout->addWidget(coboBrowser);


    // autoload groupbox
    QGroupBox* autolayoutGB = new QGroupBox(
            tr("Program start and quit"));
    autolayoutGB->setObjectName("autolayoutGB");
    tabL->addWidget(autolayoutGB);
    QVBoxLayout* autoGBLayout = new QVBoxLayout();
    autoGBLayout->setSpacing(6);
    autolayoutGB->setLayout(autoGBLayout);

    // line with full sreen option
    cbFullScreen = new QCheckBox(tr("Start in &full screen view"),
            autolayoutGB);
    autoGBLayout->addWidget(cbFullScreen);

    // line with autosave option
    cbAutosave = new QCheckBox(tr("&Save active layout on program exit "
                "without check-back"), autolayoutGB);
    autoGBLayout->addWidget(cbAutosave);

    // line with radiobutton and choose button
    QHBoxLayout* chooseLayout = new QHBoxLayout();
    autoGBLayout->addLayout(chooseLayout);
    cbAutoload = new QCheckBox(tr("&Load this layout on program startup:"),
            autolayoutGB);
    chooseLayout->addWidget(cbAutoload);
    connect(cbAutoload, SIGNAL(toggled(bool)),
            this, SLOT(slotAutoloadToggled(bool)));
    chooseLayout->addStretch();
    buttGetAutofile = new QPushButton(tr("C&hoose..."), autolayoutGB);
    chooseLayout->addWidget(buttGetAutofile);
    connect(buttGetAutofile, SIGNAL(clicked()),
            this, SLOT(slotGetAutofile()));

    // line with lineedit
    leAutoload = new QLineEdit(autolayoutGB);
    autoGBLayout->addWidget(leAutoload);


    // statusline groupbox
    QGroupBox* statuslineGB = new QGroupBox(tr("Statusline"));
    statuslineGB->setObjectName("statuslineGB");
    tabL->addWidget(statuslineGB);
    QVBoxLayout* statuslineGBL = new QVBoxLayout();
    statuslineGBL->setSpacing(6);
    statuslineGB->setLayout(statuslineGBL);

    cbShowTime = new QCheckBox(
            tr("Sho&w time for incoming and outgoing messages"),
            statuslineGB);
    statuslineGBL->addWidget(cbShowTime);

    cbConvertTime = new QCheckBox(tr("&Convert SRCP 0.8 server time "
                "human readable"),
            statuslineGB);
    statuslineGBL->addWidget(cbConvertTime);


    // spacer to push group boxes to top
    tabL->addStretch();

    tabWidget->addTab(w, tr("&Layout"));
}


void optionsDialog::setupElementTab()
{
    QWidget *w = new QWidget();
    QVBoxLayout* tabL = new QVBoxLayout(w);
    tabL->setSpacing(10);


    // special effects groupbox
    QGroupBox* effectsBG = new QGroupBox(tr("Special effects"));
    effectsBG->setObjectName("effectsBG");
    tabL->addWidget(effectsBG);
    QVBoxLayout* effectsiGbL = new QVBoxLayout();

    cbShowHp2 = new QCheckBox(tr("Show &orange light for signals"
                " switched to Hp2"), effectsBG);
    effectsiGbL->addWidget(cbShowHp2);

    cbShowBlinkingTurnouts = new QCheckBox(tr("Show &blinking turnouts"),
                effectsBG);
    effectsiGbL->addWidget(cbShowBlinkingTurnouts);

    allwaysSendState = new QCheckBox(tr("A&llways send solenoid states"
                " on routing"), effectsBG);
    effectsiGbL->addWidget(allwaysSendState);
    effectsBG->setLayout(effectsiGbL);


    // bubblehelp groupbox
    QGroupBox* bubblehelpBG = new QGroupBox(tr("Bubblehelp"));
    bubblehelpBG->setObjectName("bubblehelpBG");
    tabL->addWidget(bubblehelpBG);

    QVBoxLayout* bubblehelpGbL = new QVBoxLayout();

    cbGenBubble = new QCheckBox(tr("Globally show help &bubbles"),
            bubblehelpBG);
    bubblehelpGbL->addWidget(cbGenBubble);
    connect(cbGenBubble, SIGNAL(toggled(bool)),
            this, SLOT(globalBubbleHelpChanged(bool)));

    cbDataBubble = new QCheckBox(tr("Show help bubbles for element &data"),
            bubblehelpBG);
    bubblehelpGbL->addWidget(cbDataBubble);
    bubblehelpBG->setLayout(bubblehelpGbL);


    // text groupbox
    QGroupBox* soladdrBG = new QGroupBox(tr("Solenoid labeling"));
    soladdrBG->setObjectName("soladdrBG");
    tabL->addWidget(soladdrBG);

    QVBoxLayout* soladdrBgL = new QVBoxLayout();
    rbShowAddr = new QRadioButton(tr("Show decoder &address"),
            soladdrBG);
    soladdrBgL->addWidget(rbShowAddr);
    rbShowTxt = new QRadioButton(
            tr("Show &text (turnout or signal name)"), soladdrBG);
    soladdrBgL->addWidget(rbShowTxt);
    soladdrBG->setLayout(soladdrBgL);


    // init groupbox
    QGroupBox* initsigBG = new QGroupBox(
            tr("Initialize signals on startup"));
    initsigBG->setObjectName("initsigBGB");
    tabL->addWidget(initsigBG);

    QVBoxLayout* initsigBgL = new QVBoxLayout();
    rbSignalRed =
        new QRadioButton(tr("Always on &Halt (Hp0/Hp00/Sh0)"), initsigBG);
    initsigBgL->addWidget(rbSignalRed);
    rbSignalLay =
        new QRadioButton(tr("As &saved from previous session"), initsigBG);
    initsigBgL->addWidget(rbSignalLay);
    initsigBG->setLayout(initsigBgL);


    // spacer to push group boxes to top
    tabL->addStretch();

    tabWidget->addTab(w, tr("&Elements"));
}


void optionsDialog::setupDigitalTab()
{
    QWidget *w = new QWidget();
    QVBoxLayout* tabL = new QVBoxLayout();
    tabL->setSpacing(10);
    w->setLayout(tabL);


    // protocol groupbox
    QGroupBox *protocolGB = new QGroupBox(tr("Default protocol"));
    tabL->addWidget(protocolGB);
    QVBoxLayout* protocolL = new QVBoxLayout();
    protocolGB->setLayout(protocolL);

    QButtonGroup* protocolBG = new QButtonGroup(protocolGB);
    protocolBG->setExclusive(true);

    rbProtMS = new QRadioButton("Märklin/M&otorola");
    protocolL->addWidget(rbProtMS);
    protocolBG->addButton(rbProtMS, 0);

    rbProtNA = new QRadioButton("&NMRA/DCC");
    protocolL->addWidget(rbProtNA);
    protocolBG->addButton(rbProtNA, 1);

    rbProtPS = new QRadioButton("&Protocol by Server");
    protocolL->addWidget(rbProtPS);
    protocolBG->addButton(rbProtPS, 2);

    rbProtSE = new QRadioButton("Selectri&x");
    protocolL->addWidget(rbProtSE);
    protocolBG->addButton(rbProtSE, 3);


    // solenoid groupbox
    QGroupBox* solenoidGB = new QGroupBox(tr("Solenoid defaults"));
    tabL->addWidget(solenoidGB);
    QVBoxLayout* solenoidGBL = new QVBoxLayout();
    solenoidGBL->setSpacing(6);
    solenoidGB->setLayout(solenoidGBL);

    // line with activation time
    QHBoxLayout* atimeLayout = new QHBoxLayout();
    solenoidGBL->addLayout(atimeLayout);
    QLabel* label = new QLabel(tr("Default &activation time (ms):"), solenoidGB);
    atimeLayout->addWidget(label);
    atimeLayout->addStretch();
    // 20 ms steps
    sbActiveTime = new QSpinBox(solenoidGB);
    sbActiveTime->setMinimum(50);
    sbActiveTime->setMaximum(2000);
    sbActiveTime->setSingleStep(50);
    label->setBuddy(sbActiveTime);
    sbActiveTime->setWrapping(true);
    atimeLayout->addWidget(sbActiveTime);

    // line with delay time
    QHBoxLayout* delayLayout = new QHBoxLayout();
    solenoidGBL->addLayout(delayLayout);
    label = new QLabel(tr("Routing &delay per element (ms):"), solenoidGB);
    delayLayout->addWidget(label);
    delayLayout->addStretch();
    // 50 ms steps
    sbRoutingTime = new QSpinBox(solenoidGB);
    sbRoutingTime->setMinimum(20);
    sbRoutingTime->setMaximum(1000);
    sbRoutingTime->setSingleStep(20);
    sbRoutingTime->setWrapping(true);
    label->setBuddy(sbRoutingTime);
    delayLayout->addWidget(sbRoutingTime);


    // turntable groupbox
    QGroupBox* ttGB = new QGroupBox(tr("Turntable defaults"));
    tabL->addWidget(ttGB);
    QVBoxLayout* ttGBL = new QVBoxLayout();
    ttGBL->setSpacing(6);
    ttGB->setLayout(ttGBL);

    // line with auto tt
    cbAutoTTDir = new QCheckBox(tr("Choose digital &turntable"
                " moving-direction automatically"), ttGB);
    ttGBL->addWidget(cbAutoTTDir);

    // line with tt time
    QHBoxLayout* ttLayout = new QHBoxLayout();
    ttGBL->addLayout(ttLayout);
    label = new QLabel(tr("T&ime for a 360° turn of turntable (s.ms):"), ttGB);
    ttLayout->addWidget(label);
    ttLayout->addStretch();
    leTTRoundTime = new QLineEdit(ttGB);
    leTTRoundTime->setMaxLength(6);
    leTTRoundTime->setMaximumWidth(60); //MAGIC
    label->setBuddy(leTTRoundTime);
    ttLayout->addWidget(leTTRoundTime);

    // spacer to push group boxes to top
    tabL->addStretch();

    tabWidget->addTab(w, tr("&Digital Data"));
}


void optionsDialog::setupFeedbackTab()
{
    QWidget *w = new QWidget();
    QVBoxLayout* tabL = new QVBoxLayout(w);
    tabL->setSpacing(10);

    // left column
    // 1. feedback module groupbox
    QGroupBox* inputsGB = new QGroupBox(tr("Module inputs"));
    tabL->addWidget(inputsGB);
    QVBoxLayout* inputsL = new QVBoxLayout();
    inputsGB->setLayout(inputsL);
    QButtonGroup* inputsBG = new QButtonGroup(inputsGB);

    rb16inputs = new QRadioButton(tr("1&6 Inputs"), inputsGB);
    inputsL->addWidget(rb16inputs);
    inputsBG->addButton(rb16inputs, 0);

    rb8inputs = new QRadioButton(tr("&8 Inputs"), inputsGB);
    inputsL->addWidget(rb8inputs);
    inputsBG->addButton(rb8inputs, 1);

    connect(inputsBG, SIGNAL(buttonClicked(int)),
            this, SLOT(slotLimitModules(int)));

    // Feedback bus numbering
    // * SRCP 0.7 -> number is fixed
    // * SRCP 0.8 -> number is variable
    // 3. modules per bus groupbox
    QGroupBox* busGB = new QGroupBox(tr("Connected modules per bus"));
    tabL->addWidget(busGB);
    QHBoxLayout* busGBL = new QHBoxLayout();
    busGBL->setSpacing(6);
    busGB->setLayout(busGBL);

    //Rows, Columns
    QGridLayout* busLayout = new QGridLayout();
    busLayout->setObjectName("busLayout");
    /*TODO busLayout->addColSpacing(2, 10);*/
    busGBL->addLayout(busLayout);

    busGBL->addStretch();

    // 1. line
    QLabel* label = new QLabel(tr("&Bus:"));
    busLayout->addWidget(label, 0, 0);
    bus1LE = new QLineEdit("1");
    bus1LE->setMaximumWidth(30);
    bus1LE->setMaxLength(3);
    busLayout->addWidget(bus1LE, 0, 1);
    label->setBuddy(bus1LE);

    label = new QLabel(tr("&Modules:"));
    busLayout->addWidget(label, 0, 3);
    sbFBmod_1 = new QSpinBox();
    sbFBmod_1->setMinimum(0);
    sbFBmod_1->setMaximum(31);
    busLayout->addWidget(sbFBmod_1, 0, 4);
    label->setBuddy(sbFBmod_1);
    sbFBmod_1->setMaximumWidth(38); //MAGIC
    sbFBmod_1->setWrapping(true);

    // 2. line
    label = new QLabel(tr("&Bus:"));
    busLayout->addWidget(label, 1, 0);
    bus2LE = new QLineEdit("2");
    bus2LE->setMaximumWidth(30);
    bus2LE->setMaxLength(3);
    busLayout->addWidget(bus2LE, 1, 1);
    label->setBuddy(bus2LE);

    label = new QLabel(tr("&Modules:"));
    busLayout->addWidget(label, 1, 3);
    sbFBmod_2 = new QSpinBox();
    sbFBmod_2->setMinimum(0);
    sbFBmod_2->setMaximum(31);
    busLayout->addWidget(sbFBmod_2, 1, 4);
    label->setBuddy(sbFBmod_2);
    sbFBmod_2->setMaximumWidth(38); //MAGIC
    sbFBmod_2->setWrapping(true);

    // 3. line
    label = new QLabel(tr("&Bus:"));
    busLayout->addWidget(label, 2, 0);
    bus3LE = new QLineEdit("3");
    bus3LE->setMaximumWidth(30);
    bus3LE->setMaxLength(3);
    busLayout->addWidget(bus3LE, 2, 1);
    label->setBuddy(bus3LE);

    label = new QLabel(tr("&Modules:"));
    busLayout->addWidget(label, 2, 3);
    sbFBmod_3 = new QSpinBox();
    sbFBmod_3->setMinimum(0);
    sbFBmod_3->setMaximum(31);
    busLayout->addWidget(sbFBmod_3, 2, 4);
    label->setBuddy(sbFBmod_3);
    sbFBmod_3->setMaximumWidth(38); //MAGIC
    sbFBmod_3->setWrapping(true);

    // 4. line
    label = new QLabel(tr("&Bus:"));
    busLayout->addWidget(label, 3, 0);
    bus4LE = new QLineEdit("4");
    bus4LE->setMaximumWidth(30);
    bus4LE->setMaxLength(3);
    busLayout->addWidget(bus4LE, 3, 1);
    label->setBuddy(bus4LE);

    label = new QLabel(tr("&Modules:"));
    busLayout->addWidget(label, 3, 3);
    sbFBmod_4 = new QSpinBox();
    sbFBmod_4->setMinimum(0);
    sbFBmod_4->setMaximum(31);
    busLayout->addWidget(sbFBmod_4, 3, 4);
    label->setBuddy(sbFBmod_4);
    sbFBmod_4->setMaximumWidth(38); //MAGIC
    sbFBmod_4->setWrapping(true);

    // spacer to push group boxes to top
    tabL->addStretch();

    tabWidget->addTab(w, tr("&Feedback modules"));
}


void optionsDialog::setupFeedbackTypeTab()
{
    QWidget *tab = new QWidget();
    QVBoxLayout *layout = new QVBoxLayout(tab);
    layout->setObjectName("layoutFbTt");
    layout->setMargin(10);
    layout->setSpacing(10);

    // 1. feedback module type groupbox
    QGroupBox* feedbackTypeGB = new QGroupBox(tr("Module type"));
    QVBoxLayout* feedbackTypeL = new QVBoxLayout();
    feedbackTypeL->setObjectName("feedbackTypeL");
    feedbackTypeGB->setLayout(feedbackTypeL);
    layout->addWidget(feedbackTypeGB);

    feedbackTypeBG = new QButtonGroup(feedbackTypeGB);

    QRadioButton* fbtRB = new QRadioButton(tr("S88 via &DDL"));
    feedbackTypeL->addWidget(fbtRB);
    feedbackTypeBG->addButton(fbtRB, 0);

    fbtRB = new QRadioButton(tr("i8&255-Card"));
    feedbackTypeL->addWidget(fbtRB);
    feedbackTypeBG->addButton(fbtRB, 1);

    fbtRB = new QRadioButton(tr("S88 via M60&15"));
    feedbackTypeL->addWidget(fbtRB);
    feedbackTypeBG->addButton(fbtRB, 2);

    fbtRB = new QRadioButton(tr("&Protocol by server"));
    feedbackTypeL->addWidget(fbtRB);
    feedbackTypeBG->addButton(fbtRB, 3);

    fbtRB = new QRadioButton(tr("&Selectrix"));
    feedbackTypeL->addWidget(fbtRB);
    feedbackTypeBG->addButton(fbtRB, 4);

    fbtRB->setEnabled(false);

    connect(feedbackTypeBG, SIGNAL(buttonClicked(int)),
            this, SLOT(selectFbModuleType(int)));

    // 2. selectrix init parameters groupbox
    selectrixGB = new QGroupBox(tr("Selectrix Initialization"));
    QHBoxLayout* sxinitGBL = new QHBoxLayout();
    sxinitGBL->setObjectName("sxinitGBL");
    sxinitGBL->setSpacing(10);
    selectrixGB->setLayout(sxinitGBL);
    layout->addWidget(selectrixGB);

    // 2a: parameter list
    QTreeWidget* selectrixLV = new QTreeWidget();
    selectrixLV->setAllColumnsShowFocus(true);
    selectrixLV->setHeaderLabels(
            QStringList() << tr("Bus") << tr("Address") << tr("Number"));
    sxinitGBL->addWidget(selectrixLV);

    // 2b: vertical box layout container for buttons
    QVBoxLayout* sxinitBtnLayout = new QVBoxLayout();
    sxinitBtnLayout->setObjectName("sxinitBtnLayout");
    sxinitBtnLayout->setSpacing(6);
    sxinitGBL->addLayout(sxinitBtnLayout);

    sxinitBtnLayout->addStretch();

    QPushButton* upPB = new QPushButton(tr("&Up"));
    sxinitBtnLayout->addWidget(upPB);
    upPB->setEnabled(false);

    QPushButton* downPB = new QPushButton(tr("&Down"));
    sxinitBtnLayout->addWidget(downPB);
    downPB->setEnabled(false);

    QPushButton* editPB = new QPushButton(tr("&Edit..."));
    sxinitBtnLayout->addWidget(editPB);
    editPB->setEnabled(false);

    QPushButton* addPB = new QPushButton(tr("&Add..."));
    sxinitBtnLayout->addWidget(addPB);

    QPushButton* removePB = new QPushButton(tr("&Remove"));
    sxinitBtnLayout->addWidget(removePB);
    removePB->setEnabled(false);

    // spacer to push buttons to left
    sxinitGBL->addStretch();


    tabWidget->addTab(tab, tr("Feedback &type"));
}


void optionsDialog::setupGenericMessagesTab()
{
    QWidget *w = new QWidget();
    QVBoxLayout* tabL = new QVBoxLayout(w);
    tabL->setSpacing(10);


    // dynamic data groupbox
    QGroupBox* dynamicdataGB = new QGroupBox(tr("Dynamic CRCF Data"));
    QVBoxLayout* dynamicdataL = new QVBoxLayout();
    dynamicdataGB->setLayout(dynamicdataL);
    tabL->addWidget(dynamicdataGB);

    sroutestateCB = new QCheckBox(
            tr("Send broadcasted &route state change messages"));
    dynamicdataL->addWidget(sroutestateCB);
    sroutestateCB->setToolTip(tr(
                "Send a broadcasted state change message when a\n"
                "route is released or successfully activated. This\n"
                "is typically used for CRCF scripting."));

    strainnumberCB = new QCheckBox(
            tr("Send broadcasted &train number messages"));
    dynamicdataL->addWidget(strainnumberCB);
    strainnumberCB->setToolTip(tr(
                "Send a broadcasted generic message when a train number is\n"
                "assigned to a new route. This is used for train tracking."));

    // spacer to push group boxes to top
    tabL->addStretch();
    tabWidget->addTab(w, tr("Generic &Messages"));
}


void optionsDialog::slotGetAutofile()
{
    QString filename = QFileDialog::getOpenFileName(this,
            tr("Open layout file"), QDir::homePath(),
            QString(tr("Layouts")) + " (*" + GF_GBSEXT + ")");

    if (!filename.isEmpty())
        leAutoload->setText(filename);
}


void optionsDialog::slotAutoloadToggled(bool load)
{
    // if autoload is (de-)selected, do the same with filebutton and
    // namefield
    leAutoload->setEnabled(load);
    buttGetAutofile->setEnabled(load);
}


/*
 * depending on feedback module type (16 or 8 port) set the spin box
 * ranges, 16 <-> 0, 8 <-> 1
 */
void optionsDialog::slotLimitModules(int iType_)
{
    sbFBmod_1->setRange(0, 31 + iType_ * 31);
    sbFBmod_2->setRange(0, 31 + iType_ * 31);
    sbFBmod_3->setRange(0, 31 + iType_ * 31);
    sbFBmod_4->setRange(0, 31 + iType_ * 31);
}

/* enable Selectrix configuration if choosen */
void optionsDialog::selectFbModuleType(int no)
{
    if (no == 4)
        selectrixGB->setEnabled(true);
    else
        selectrixGB->setEnabled(false);
}

// TODO: remove this
bool optionsDialog::valuesAreValid()
{
    QString sText;

    sText = leAutoload->text();
    if (sText == "" && leAutoload->isEnabled()) {
        qApp->beep();
        QMessageBox::warning(this,
                             tr("Filename missing"),
                             tr("You have enabled the autoloader,\n"
                                "so you must select a filename, too.\n"),
                             tr("&OK"), 0, 0, 0, 0);
        // butt 1: OK, butt 2+3: not avail.
        // <ENTER> + <ESC> default to butt 0 = OK
        buttGetAutofile->setFocus();
        return false;
    }

    sText = leTTRoundTime->text();
    if (sText == "") {
        qApp->beep();
        QMessageBox::warning(this,
                             tr("Turntable at lightspeed"),
                             tr("Please specify the total time\n"
                                "that a turntable needs for a whole\n"
                                "360° turn.\n"),
                             tr("&OK"), 0, 0, 0, 0);
        // butt 1: OK, butt 2+3: not avail.
        // <ENTER> + <ESC> default to butt 0 = OK
        leTTRoundTime->setFocus();
        return false;
    }

    if (sbFBmod_1->value() == 0 && sbFBmod_2->value() == 0 &&
        sbFBmod_3->value() == 0 && sbFBmod_4->value() == 0) {
        qApp->beep();
        QMessageBox::information(this,
                                 tr("Feedback modules missing"),
                                 tr
                                 ("On every feedback bus there seem to be\n"
                                  "no feedback modules.\n\n"
                                  "I assume that at least 1 module\n"
                                  "is connected to bus #1."), tr("&Yes"),
                                 0, 0, 0, 0);
        // butt 1: Yes, butt 2+3: N/A.
        // <ENTER> + <ESC> default to butt 0 = OK
        sbFBmod_1->setValue(1);
        return true;
    }

    // everything is correct, so return VALID
    return true;
}


void optionsDialog::getPreferences(Preferences& prf)
{
    // Layout Page
    prf.layoutcols = sbDefaultCols->value();
    prf.layoutrows = sbDefaultRows->value();

    prf.editor = coboEditor->currentText();
    prf.browser = coboBrowser->currentText();

    prf.fullscreen = cbFullScreen->isChecked();
    prf.autosave = cbAutosave->isChecked();
    prf.autoload = cbAutoload->isChecked();
    prf.autolayout = leAutoload->text();

    prf.converttime = cbConvertTime->isChecked();
    prf.showtime = cbShowTime->isChecked();

    // Element Page
    prf.hp2 = cbShowHp2->isChecked();
    prf.blinkingturnouts = cbShowBlinkingTurnouts->isChecked();
    prf.sendstate = allwaysSendState->isChecked();
    prf.datatooltips = cbDataBubble->isChecked();
    prf.addresslabeling = rbShowAddr->isChecked();
    prf.initsignalsred = rbSignalRed->isChecked();

    // Digital Page
    if (rbProtMS->isChecked())
        prf.protocol = 1;
    else if (rbProtNA->isChecked())
        prf.protocol = 0;
    else if (rbProtSE->isChecked())
        prf.protocol = 3;
    else
        prf.protocol = 2;
    prf.activetime = sbActiveTime->value();
    prf.routingtime = sbRoutingTime->value();

    prf.autottdir = cbAutoTTDir->isChecked();
    prf.ttroundtime = leTTRoundTime->text().toDouble();

    // Feedback Page
    if (rb16inputs->isChecked())
        prf.fbfactor = 0;
    else
        prf.fbfactor = 1;

    prf.fbbus1.modules = sbFBmod_1->value();
    prf.fbbus2.modules = sbFBmod_2->value();
    prf.fbbus3.modules = sbFBmod_3->value();
    prf.fbbus4.modules = sbFBmod_4->value();
    prf.fbbus1.number = bus1LE->text().toUInt();
    prf.fbbus2.number = bus2LE->text().toUInt();
    prf.fbbus3.number = bus3LE->text().toUInt();
    prf.fbbus4.number = bus4LE->text().toUInt();

    // Feedback Type Page
    prf.fbmoduletype = feedbackTypeBG->checkedId();
    // TODO: Selectrix stuff

    // Generic Messages Page
    prf.gmbroadcastroutestate = sroutestateCB->isChecked();
    prf.gmbroadcasttrainnumber = strainnumberCB->isChecked();
}


void optionsDialog::setPreferences(const Preferences& prf)
{
    // Layout Page
    sbDefaultCols->setValue(prf.layoutcols);
    sbDefaultRows->setValue(prf.layoutrows);

    bool found = false;
    for (int i = 0; i < coboEditor->count(); i++) {
        if (coboEditor->itemText(i) == prf.editor) {
            coboEditor->setCurrentIndex(i);
            found = true;
            break;
        }
    }
    if (!found) {
        //TODO: prog == ""
        coboEditor->addItem(prf.editor);
        coboEditor->setCurrentIndex(coboEditor->count() - 1);
    }

    found = false;
    for (int i = 0; i < coboBrowser->count(); i++) {
        if (coboBrowser->itemText(i) == prf.browser) {
            coboBrowser->setCurrentIndex(i);
            found = true;
            break;
        }
    }
    if (!found) {
        //TODO: prog == ""
        coboBrowser->addItem(prf.browser);
        coboBrowser->setCurrentIndex(coboBrowser->count() - 1);
    }

    cbFullScreen->setChecked(prf.fullscreen);
    cbAutosave->setChecked(prf.autosave);
    cbAutoload->setChecked(prf.autoload);
    if (prf.autoload)
        leAutoload->setText(prf.autolayout);
    slotAutoloadToggled(prf.autoload);

    cbConvertTime->setChecked(prf.converttime);
    cbShowTime->setChecked(prf.showtime);

    // Element Page
    cbShowHp2->setChecked(prf.hp2);
    cbShowBlinkingTurnouts->setChecked(prf.blinkingturnouts);
    allwaysSendState->setChecked(prf.sendstate);
    /*TODO: cbGenBubble->setChecked(QToolTip::isGloballyEnabled());*/
    globalBubbleHelpChanged(cbGenBubble->isChecked());
    cbDataBubble->setChecked(prf.datatooltips);

    if (prf.addresslabeling)
        rbShowAddr->setChecked(true);
    else
        rbShowTxt->setChecked(true);

    if (prf.initsignalsred)
        rbSignalRed->setChecked(true);
    else
        rbSignalLay->setChecked(true);

    // Digital Page
    switch (prf.protocol) {
        case 0:
            rbProtNA->setChecked(true);
            break;
        case 1:
            rbProtMS->setChecked(true);
            break;
        case 2:
            rbProtPS->setChecked(true);
            break;
        case 3:
            rbProtSE->setChecked(true);
            break;
    }

    cbAutoTTDir->setChecked(prf.autottdir);
    sbActiveTime->setValue(prf.activetime);
    sbRoutingTime->setValue(prf.routingtime);

    QString t;
    t.sprintf("%.2f", prf.ttroundtime);
    leTTRoundTime->setText(t);

    // Feedback Page
    if (prf.fbfactor == 0) {
        rb16inputs->setChecked(true);
        slotLimitModules(0);
    }
    else {
        rb8inputs->setChecked(true);
        slotLimitModules(1);
    }

    bus1LE->setText(QString::number(prf.fbbus1.number));
    sbFBmod_1->setValue(prf.fbbus1.modules);
    bus2LE->setText(QString::number(prf.fbbus2.number));
    sbFBmod_2->setValue(prf.fbbus2.modules);
    bus3LE->setText(QString::number(prf.fbbus3.number));
    sbFBmod_3->setValue(prf.fbbus3.modules);
    bus4LE->setText(QString::number(prf.fbbus4.number));
    sbFBmod_4->setValue(prf.fbbus4.modules);

    // Feedback type Page
    QAbstractButton* fbtbtn = feedbackTypeBG->button(prf.fbmoduletype);
    if (fbtbtn != NULL)
        fbtbtn->setChecked(true);
    else
        qWarning("Unexpected feedback type: %d", prf.fbmoduletype);

    selectFbModuleType(prf.fbmoduletype);

    // Generic Messages Page
    sroutestateCB->setChecked(prf.gmbroadcastroutestate);
    strainnumberCB->setChecked(prf.gmbroadcasttrainnumber);
}

/*
 * show response if global bubble help ist enabled/disabled
 */
void optionsDialog::globalBubbleHelpChanged(bool enabled)
{
    cbDataBubble->setEnabled(enabled);
}
