/***************************************************************************
                           router.h
                           version 0.5.3 $Revision: 1.33 $
                           -------------------------------
    copyright            : (C) 2004-2008 by Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2008-10-07 17:34:43 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
  This code implements the router object. This object cares for the routing
  list, loads it from file, save it to file, shows routing edit window.
 ***************************************************************************/

#ifndef ROUTER_H
#define ROUTER_H

#include <QVector>

#include "crcfmessage.h"
#include "route.h"
#include "srcpmessage.h"


// forward declaration
class SpdrPanel;

class Router: public QObject
{
    Q_OBJECT

public:
    Router(QObject* parent = NULL, const char* name = NULL);
    Router(QObject* parent = NULL, QVector<SpdrPanel*>* elPtr = NULL,
            const char* name = NULL);
    ~Router();
    void readFileTextFromStream(QTextStream&);
    void writeFileTextToStream(QTextStream&);
    unsigned int getRouteCount();
    Route* addNewRoute();
    Route* getRouteAt(unsigned int);
    void deleteRouteAt(unsigned int);
    void deleteRoute(Route*);
    void copyRouteAt(unsigned int);
    Route* copyRoute(Route*);
    bool editRouteAt(QWidget*, int);
    bool editRoute(QWidget*, Route*);
    bool isModified();
    void clear();
    void selectedRouteChanged(Route*);
    void showRouteAt(int);
    void startRecordModeAt(unsigned int);
    void activateRoute(Route*);
    void activateRouteAt(unsigned int);
    void releaseRoute(Route*);
    void releaseRouteAt(unsigned int);
    void withdrawRoute(Route*);
    void setInfoSessionId(unsigned int);
    void setServerHasGm(bool);
    bool serverHasGm();
    void handleCrcfTrainMessage(unsigned int, const CrcfMessage*);
    void processGenericMessage(unsigned int, unsigned int,
            const CrcfMessage*);

public slots:
    void clearRoutes();
    void processRouteState(Route*, int);
    void recordElement(element*, elemRecordType);
    void resetRoute(element*, GbsButtonState);
    void resetSelectedSignal();
    void setRoute(element*, GbsButtonState, GbsButtonState);
    void switchVisualMode(elemVisualMode);
    void unlockAllLockedRoutes();
    void feedbackPortChanged(unsigned int, unsigned int, bool);
    void changeTrainNumber(unsigned int, unsigned int);
    void sendTrainNumbers();

private:
    Route* selectedRoute;
    Route* resetRt;
    element* selectedStartSig;
    bool modified;
    bool serverhasgm;
    elemVisualMode visualmode;
    GbsButtonState lastcb;
    unsigned int infosessionid;
    QVector<SpdrPanel*>* gbsElements;
    QList<Route*> routeList;

    void setupRouteElements();
    void updateRouteElements();
    Route* getRouteWithId(unsigned int);
    //Route* getRouteWithSection(unsigned int);
    //Route* getRouteWithTrain(unsigned int);
    Route* getReservedRouteWithExitSignal(element*);
    Route* getLockedRouteWithEntrySignal(element*);
    Route* getUnlockedRouteWithEntrySignal(element*, GbsButtonState,
            GbsButtonState);
    Route* getUnlockedRouteWithExitSignal(element*, GbsButtonState,
            GbsButtonState);
    Route* getRouteUsedByTrain(unsigned int);
    Route* getUtilizedRouteToSignal(element*);
    Route* getUtilizedRouteToTrack(unsigned int);
    Route* getUtilizedRouteToRoute(Route*);
    void transferTrainNumber(Route*);
    unsigned int getMaximumRouteIdNumber();
    void sendGmCrcfMessage(unsigned int, unsigned int, const QString&);

signals:
    void getElementByAddress(const int, const int, element**);
    void routeFunctionFinished();
    void routeDataChanged(Route*);
    void routeListChanged();
    void routeStateChanged(Route*);
    void sendSrcpMessage(SrcpMessage*);
    void statusMessage(const QString&);
    void startRouteTimer(Route::RouteType);
    void updateRoutePathLEDs(const stateElement&, const stateElement&,
            Route::RouteSetAction&);
};
#endif // ROUTER_H

