/*
 * serverinfodialog.cpp
 * --------------------
 * Begin        : 2007-08-22
 * Last modified: $Date: 2009-10-27 20:29:24 $
 *                $Revision: 1.8 $
 * Copyright    : (C) 2007-2008 by Guido Scholz
 * E-Mail       : guido.scholz@bayernline.de
 * Description  : Dialog window to display SRCP server information
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QGroupBox>
#include <QPushButton>
#include <QLayout>

#include "serverinfodialog.h"


ServerInfoDialog::ServerInfoDialog(
        QWidget* parent, const char* name) : QDialog(parent)
{
    setObjectName(name);
    setWindowTitle(tr("SRCP server information"));
    QVBoxLayout* baseLayout = new QVBoxLayout();
    //baseLayout->setSpacing(10);
    setLayout(baseLayout);

    // command session groupbox
    QGroupBox* commandGB = new QGroupBox(tr("Command session"));
    baseLayout->addWidget(commandGB);

    // grid layout with 4 rows and 2 columns
    QGridLayout* cmdLayout = new QGridLayout(commandGB);

    // 1. line
    cmdLayout->addWidget(new QLabel(tr("Server:")), 0, 0);
    cmdServerLbl = new QLabel("");
    cmdLayout->addWidget(cmdServerLbl, 0, 1);

    // 2. line
    cmdLayout->addWidget(new QLabel(tr("SRCP version:")), 1, 0);
    cmdSrcpLbl = new QLabel("");
    cmdLayout->addWidget(cmdSrcpLbl, 1, 1);

    QLabel* label;
    // 3. line
    label = new QLabel(tr("Other SRCP version:"));
    cmdLayout->addWidget(label, 2, 0);
    cmdSrcpOtherLbl = new QLabel("");
    cmdLayout->addWidget(cmdSrcpOtherLbl, 2, 1);

    // 4. line
    label = new QLabel(tr("SRCP session id:"));
    cmdLayout->addWidget(label, 3, 0);
    cmdSessionIdLbl = new QLabel("");
    cmdLayout->addWidget(cmdSessionIdLbl, 3, 1);

    // 5. line
    label = new QLabel(tr("Generic Messages:"));
    cmdLayout->addWidget(label, 4, 0);
    cmdGmLbl = new QLabel("No");
    cmdLayout->addWidget(cmdGmLbl, 4, 1);


    // info session groupbox
    QGroupBox* infoGB = new QGroupBox(tr("Info session"));
    baseLayout->addWidget(infoGB);
    QGridLayout* infoLayout = new QGridLayout(infoGB);

    // 1. line
    infoLayout->addWidget(new QLabel(tr("Server:")), 0, 0);
    infoServerLbl = new QLabel("");
    infoLayout->addWidget(infoServerLbl, 0, 1);

    // 2. line
    infoLayout->addWidget(new QLabel(tr("SRCP version:")), 1, 0);
    infoSrcpLbl = new QLabel("");
    infoLayout->addWidget(infoSrcpLbl, 1, 1);

    // 3. line
    infoLayout->addWidget(new QLabel(tr("Other SRCP version:")), 2, 0);
    infoSrcpOtherLbl = new QLabel("");
    infoLayout->addWidget(infoSrcpOtherLbl, 2, 1);

    // 4. line
    infoLayout->addWidget(new QLabel(tr("SRCP session id:")) , 3, 0);
    infoSessionIdLbl = new QLabel("");
    infoLayout->addWidget(infoSessionIdLbl, 3, 1);


    // OK button
    QHBoxLayout* buttonLayout = new QHBoxLayout();
    buttonLayout->setSpacing(6);
    baseLayout->addLayout(buttonLayout);
    buttonLayout->addStretch();

    QPushButton* okPB = new QPushButton(tr("OK"));
    buttonLayout->addWidget(okPB);
    okPB->setDefault(true);
    connect(okPB, SIGNAL(clicked()), this, SLOT(accept()));

    buttonLayout->addStretch();
}

/*
 * set data of command session
 */
void ServerInfoDialog::setCommandSessionData(const QString& server,
        const QString& srcp, const QString& srcpother,
        unsigned int sessionid, bool gm)
{
    cmdServerLbl->setText(server);
    cmdSrcpLbl->setText(srcp);
    cmdSrcpOtherLbl->setText(srcpother);
    cmdSessionIdLbl->setText(QString::number(sessionid));

    if (gm)
        cmdGmLbl->setText(tr("Yes"));
    else
        cmdGmLbl->setText(tr("No"));
}

/*
 * set data of info session
 */
void ServerInfoDialog::setInfoSessionData(const QString& server,
        const QString& srcp, const QString& srcpother,
        unsigned int sessionid)
{
    infoServerLbl->setText(server);
    infoSrcpLbl->setText(srcp);
    infoSrcpOtherLbl->setText(srcpother);
    infoSessionIdLbl->setText(QString::number(sessionid));
}

